/*
* Copyright (c) 2008 - 2012  Extensoft, Inc.
* All Rights Reserved.
*
* For redistribution and licensing information please visit: http://www.artisteer.com/?p=code_license_agreement
* For support and help, please visit our developer's section: http://www.artisteer.com/?p=dev
*
*/
/*jshint forin:true, noarg:true, noempty:true, eqeqeq:true, bitwise:true, strict:true, undef:true, curly:false, browser:true, jquery:true, devel:true */
/*global content, design, template, result, options*/

function build() {
    'use strict';
    if ($('.art-sidebar1,.art-sidebar2').length === 0) {
        var readmeMessage = 'Your theme MUST contain at least one sidebar, otherwise it cannot be used with Moodle.';
        result.write('readme.txt', readmeMessage); 
        return;
    }
    var fragments = initFragmentsVariables();
    initTemplateFiles(fragments);
    getHeadLinks(fragments);
    getBlock(fragments);
    getMetadataHeader(fragments);
    getPageLayout(fragments);
    processPlaceholders(fragments);
    processPathes(fragments);
}
function getHeadLinks(fragments) {
    'use strict';
    fragments.headLinks = '';
    fragments.styleSheets = "'general',";
    fragments.javascripts = '';

    var scripts = '';
    var styles = '';
    var headNode = '';
    jQuery('head').clone().find('meta[charset],title,link[href$="style.css"]').remove().end().contents().filter(function () { return 3 !== this.nodeType; }).each(function () {
        if (8 === this.nodeType) {
            var expression = this.nodeValue.match(/\[if (.*)\]>/);
            var tag = this.nodeValue.match(/>(.*)</);
            if (expression && tag && 'link' === jQuery(tag[1])[0].tagName.toLowerCase()) {
                var name = jQuery(tag[1]).attr('href').split('/').pop();
                fragments.headLinks += '<!--[if ' + expression[1] + ']>' + tag[1].replace(name, "<?php echo $CFG->wwwroot . '/theme/' . $PAGE->theme->name . '/style/" + name + "';?>") + '<![endif]-->\n';
            } else {
                headNode += '<!--' + this.nodeValue + '-->\n';
            }
        } else {
            if ('link' === this.tagName.toLowerCase() && $(this).attr('href') && -1 === $(this).attr('href').indexOf("http")) {
                var name1 = $(this).attr('href').split('/').pop();
                fragments.styleSheets += "'" + name1.substr(0, name1.toString().lastIndexOf('.')) + "',";
            } else if ('script' === this.tagName.toLowerCase() && $(this).attr('src')) {
                var name2 = $(this).attr('src').split('/').pop();
                fragments.javascripts += "'" + name2.substr(0, name2.toString().lastIndexOf('.')) + "',";
            } else {
                var nodeValue = $(this).outerHTML();
                if (-1 === nodeValue.indexOf('/>')) {
                    nodeValue = nodeValue.substr(0, nodeValue.length - 1) + '/>';
                }
                headNode += nodeValue + '\n';
            }
        }
    });

    fragments.styleSheets = fragments.styleSheets.substr(0, fragments.styleSheets.lastIndexOf(','));
    fragments.javascripts = fragments.javascripts.substr(0, fragments.javascripts.lastIndexOf(','));
    fragments.headLinks += headNode;
}

function initFragmentsVariables() {
    'use strict';
    var themeTags, sheetWidth, i, fragments = { };
    template.enumerateRecursive('/fragments/').forEach(function(filename) {
        fragments[filename.split('/').pop().split('.').shift()] = template.read(filename);
    });
    template.remove('/fragments/');
    
    $.each(options, function (name, value) {
        fragments['option' + name] = value;
    });
    
    
    themeTags = options.TemplateTags.split(',').map($.trim);
    sheetWidth = design.read('style.css').match(/\.art-sheet\s*\{[^\}]+[^\w\-]width\s*:\s*([\d\w%]+)/)[1];
    themeTags.push(sheetWidth.indexOf('%') !== -1 ? 'flexible width' :'fixed width');
    themeTags.push('valid HTML5');
    themeTags.push('widgets');
    switch ($('.art-layout-cell').length) {
    case 1:
        themeTags.push('one column');
        break;
    case 2:
        themeTags.push('two columns');
        themeTags.push($('.art-layout-cell:eq(0)').is('.art-content') ? 'right sidebar' : 'left sidebar');
        break;
    case 3:
        themeTags.push('three columns');
        break;
    }
    themeTags.sort();
    for (i = 1; i < themeTags.length; i++) {
        if (themeTags[i] === themeTags[i - 1]) {
            themeTags.splice(i--, 1);
        }
    }
    fragments.templateTags = themeTags.join(', ');

    return fragments;
}


function initTemplateFiles(fragments) {
    'use strict';
    var styleCss, additionalCssSpliter;
    //The array of matches Artisteer and Moodle icons names
    var coreImages = { 'postauthoricon.png': 'i/roles.png', 
                       'postediticon.png': 't/edit.gif',
                       'postpdficon.png': 'f/pdf.png',
                       'postdateicon.png': 't/calendar.png',
                       'postemailicon.png': 't/email.png'
                        };
    design.enumerate('/').forEach(function (file) {
        if (!design.isFile(file) || -1 !== ['debug.html'].indexOf(file)) {
            return;
        }
        switch (file.split('.').pop()) {
            case 'js':
                design.copyTo(file, result, '/javascript/' + file);
                break;
            case 'swf':
                design.copyTo(file, result, '/pix/' + file);
                break;
            case 'css':
                if (file !== 'style.css') {
                    design.copyTo(file, result, '/style/' + file);
                }
                break;
            default: design.copyTo(file, result, file);
        }
    });
    design.enumerate('/images').forEach(function (file) {
        if (coreImages.hasOwnProperty(file)) {
            // Copy associated images to pix_core folder to override the standard icons
            design.copyTo('/images/' + file, result, '/pix_core/' + coreImages[file]);
        } else {
            design.copyTo('/images/' + file, result, '/pix/' + file);
        }
    });
    styleCss = design.read('/style.css');
    additionalCssSpliter = '/* Begin Additional CSS Styles */';
    styleCss = styleCss.split(additionalCssSpliter);
    fragments.css = styleCss[0];
    fragments.customStyle = additionalCssSpliter + styleCss[1];
    
    template.copyTo('/', result, '/');
    
    result.write('/javascript/script.js', design.read('script.js') + "\r\n" + template.read('/javascript/script.js'));
}



function getBlock(fragments) {
    'use strict';
    var block = $('.art-block').eq(0).clone(), blockheaderParts, blockcontentParts, blockParts;
    
    blockheaderParts = ($('.art-blockheader', block).addClass('header title').find('h3').html('[token]').end().outerHTML() || '[token]').split('[token]');
    fragments.block_title_head = blockheaderParts[0].replace('title">', 'title">{block_head}');
    fragments.block_title_title = ($.trim(blockheaderParts[1]) !== '' ? '\r\n{$title}\r\n' : '');
    fragments.block_title_tail = blockheaderParts[1].replace('>', '>{$controlshtml}');
    fragments.block_not_hideable = ($.trim(blockheaderParts[1]) === '' ? '$bc->collapsible = block_contents::NOT_HIDEABLE;' : '');
    
    blockcontentParts = $('.art-blockcontent', block).addClass('content').html('[token]').outerHTML().split('[token]');
    fragments.block_content_head = blockcontentParts[0].replace();
    fragments.block_content_tail = blockcontentParts[1];
    
    blockParts = $(block).addClass('contextual-links-region {$classes}').attr('token', 'token').find('.art-blockheader,.art-blockcontent').replaceWith('[token]').end().outerHTML().split('[token]');
    fragments.block_head_head = blockParts[0].replace(/token\s*=\s*['"]token['"]/, '{$attrs}');
    fragments.block_head_class = $.trim(fragments.block_title_tail) === '' ? 'header ' : '';
    fragments.block_tail = blockParts[blockParts.length - 1];
}

function getMetadataHeader(fragments) {
    'use strict';
    var articleTitle = $('article').clone().find('.art-postcontent').remove().end();
    $('.art-postmetadataheader, .art-postheadericons, .art-postfootericons, .art-postmetadatafooter, .art-postheader').remove(); // Remove Post Header And Icons
    $('.art-postheader', articleTitle).addClass('{$classes}').attr('token', 'token');

    if ($('.art-postheader', articleTitle).find('span').length > 0) {
        $('.art-postheader', articleTitle).find('span').html('{$text}');
    }else {
        $('.art-postheader', articleTitle).html('{$text}');
    }

    $('.art-postheadericons, .art-postfootericons, .art-postmetadatafooter', articleTitle).remove();
    fragments.post_heading = articleTitle.html().replace('<h2', '<h{$level}').replace('h2>', 'h{$level}>').replace(/token\s*=\s*['"]token['"]/, '{$id}');
}

function getPageLayout(fragments) {
    'use strict';
    var body = $('body').clone(), sidebarsList = [];
    
    fragments.beforeHeader = '<?php if ($hasheading) { ?>';
    fragments.afterHeader = '<?php } ?>';
    $('header', body).before('\r\n{beforeHeader}\r\n').after('\r\n{afterHeader}\r\n');
    // Export Header Search
	if ($('header form.art-search input[type="text"]', body).length > 0){
		$('header form.art-search input[type="text"]', body).attr('name', 'search');
		fragments.search_in_header = fragments.search_in_header.replace('{search_controls}', $('header form.art-search', body).html().replace(/\n/g, '').replace(/">/g, '" />'));
		$('header form.art-search', body).replaceWith('{search_in_header}');
	}
    
    fragments.headline = '<?php echo $SITE->shortname; ?>';
    $('.art-headline a').html('{headline}');
    fragments.slogan = '<?php echo $SITE->fullname; ?>';
    $('.art-slogan').html('{slogan}');
    
    fragments.beforeHmenu = '<?php if ($hascustommenu) { ?>';
    fragments.innerHmenu = '<?php echo $custommenu; ?>';
    fragments.afterHmenu = '<?php } ?>';
    $('nav', body).before('\r\n{beforeHmenu}\r\n').after('\r\n{afterHmenu}\r\n').find('.art-hmenu').replaceWith('\r\n{innerHmenu}\r\n');

    $('.art-postcontent', body).html("{postContent}");
    
    // Base theme structure imitation
    $('.art-content', body).attr('id', 'region-main');
    $('.art-sheet', body).attr('id', 'page-content');
    
    // Set href source for Headline link
    $('.art-headline a', body).attr('href', '<?php echo $CFG->wwwroot;?>');
    
    // Set styles for a container with id="page-content" 
    var artSheet = design.read('style.css').match(/.art-sheet\{*[\s\S]*?\}/g);
    fragments.page_content = '';
    for (var value in artSheet) {
        if (artSheet.hasOwnProperty(value)) {
              fragments.page_content += artSheet[value].substring(artSheet[value].indexOf('{') + 1, artSheet[value].indexOf('}'));
        }
    }

    $('.art-sidebar1,.art-sidebar2', body).each(function() {
        var position = $(this).next().length ? 'pre' : 'post';
        sidebarsList.push('"side-' + position + '"');
        fragments['before_' + position + '_sidebar'] = '<?php if ($hasside' + position + ') { ?>';
        fragments['inner_' + position + '_sidebar'] = "<?php echo $OUTPUT->blocks_for_region('side-" + position + "') ?>";
        fragments['after_' + position + '_sidebar'] = '<?php } ?>';
        $(this).before('{before_' + position + '_sidebar}');
        $(this).html('{inner_' + position + '_sidebar}');
        $(this).after('{after_' + position + '_sidebar}');
    });
    
    // Footnote text fixing
    $('#art-footnote-links a', body).filter(function () {
        return 'Web Template' === $(this).text().trim();
    }).attr({
        href: 'http://www.artisteer.com/?p=moodle_themes',
        target: '_blank'
    }).text('Moodle theme');
    
    fragments.allSidebarsConfig = sidebarsList.join(', ');
    fragments.defaultRegion  = sidebarsList[sidebarsList.length - 1];
    fragments.oneSidebarConfig = sidebarsList[0];
    var bodyHtml = $(body).html();
    // Replace paths to flash sources
    bodyHtml = bodyHtml.replace(/images\/flash.swf/g, '<?php echo $CFG->wwwroot . \'/theme/\' . $PAGE->theme->name . \'/pix/flash.swf\';?>');
    bodyHtml = bodyHtml.replace(/container.swf/g, '<?php echo $CFG->wwwroot . \'/theme/\' . $PAGE->theme->name . \'/pix/container.swf\';?>');
    bodyHtml = bodyHtml.replace(/Flash player">/g, 'Flash player" />');
    bodyHtml = bodyHtml.replace(/images\//g, '<?php echo $CFG->wwwroot . \'/theme/\' . $PAGE->theme->name . \'/pix/\';?>');
    
    fragments.pageContent = bodyHtml;
}

function processPlaceholders(fragments) {
    'use strict';
    var placeholderRegExp = /\{([\w\d_\-]+)\}/g ;
    function placeholderReplacer(match, placeholderName) {
        var newContent = fragments[placeholderName];
        if (typeof newContent !== 'undefined') {
            return String(newContent);
        } else {
            console.error('Not Found Placeholder: ' + placeholderName);
            return '';
        }
    }
    
    result.enumerateRecursive('/').forEach(function(filename) {
        var contentFile;
        if (!result.isBinary(filename)) {
            contentFile = result.read(filename);
            result.remove(filename);
            while (contentFile.search(placeholderRegExp) !== -1) {
                contentFile = contentFile.replace(placeholderRegExp, placeholderReplacer);
            }
            while (filename.search(placeholderRegExp) !== -1) {
                filename = filename.replace(placeholderRegExp, placeholderReplacer);
            }
            result.write(filename, contentFile);
        }
    });
}

function processPathes(fragments) {
    'use strict';
    
    var images = { };
    result.enumerateRecursive('/').forEach(function(filename) {
        if(filename.split('.').pop() === 'css') {
            var style = result.read(filename);
            style = style.replace(/(url\(['"]?)(.*?)(['"]?\))/g, function(str, start, name, end) {
                name = name.split('/').pop().split('.');
                name.pop();
                return start + '[[pix:theme|' + name.join('.') + ']]' + end;
            });
            result.write(filename, style);
        }
    });
}

